/*
 * Licensed to meast-summarize under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. IlluSnip licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package example;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.jgrapht.graph.DirectedMultigraph;

import name.dxliu.agent.GraphAgent;
import name.dxliu.agent.OracleAgent;
import name.dxliu.agent.RelationChecker;
import name.dxliu.associations.AssociationFinder;
import name.dxliu.associations.AssociationTree;
import name.dxliu.bean.IntegerEdge;
import name.dxliu.bean.PairBean;
import name.dxliu.example.ExampleGraphAgent;
import name.dxliu.example.ExampleOracleAgent;
import name.dxliu.example.ExampleRelationChecker;
import name.dxliu.example.Step1_ExampleTriplePreprocessor;
import name.dxliu.example.Step2_ExampleOracleUsage;
import ws.meast.summarization.mps.ClassDistance;
import ws.meast.summarization.mps.GreedyMultiPatternSelector;
import ws.meast.summarization.mps.PatternDistance;
import ws.meast.summarization.mpsimp.AStarPatternDistanceImp;
import ws.meast.summarization.mpsimp.PlainClassDistanceImp;

/**
 * 
 * @author Daxin Liu
 * This class shows how to select favorable patterns from numerous patterns.
 */
public class MainExampleGreedyMultiSelector {

	public static List<Integer> QUERYENTITIES = null;
	public static int ASSOCIATIONSIZE = 0;
	
	public static void main(String[] args) throws IOException {
		//use meats-search components to mining patterns.		
		List<PairBean> patterns = GivenListOfPatternSupporterTrees();
		
		ClassDistance classDistance = new PlainClassDistanceImp();
		PatternDistance patternDistance = new AStarPatternDistanceImp(classDistance);		
		List<PairBean> patternsSelect = GreedyMultiPatternSelector.selectByConstraint(QUERYENTITIES, patterns,5,ASSOCIATIONSIZE,0.5,patternDistance); 
		
		System.out.println(patternsSelect.toString());
	}

	/**
	 * use meats-search components to mining patterns.
	 * @return mined patterns.
	 * @throws IOException
	 */
	public static List<PairBean> GivenListOfPatternSupporterTrees() throws IOException{
		Step1_ExampleTriplePreprocessor.main(null);
		Step2_ExampleOracleUsage.main(null);
		AssociationFinder finder = new AssociationFinder();		
		//read entity-relation graph.
		DirectedMultigraph<Integer, IntegerEdge> graph = new DirectedMultigraph<>(IntegerEdge.class);
		List<String> allLine = Files.readAllLines(Paths.get("example/out_id_relation_triples"), Charset.defaultCharset());
		for(String line:allLine){
			String[] spo = line.split(" ");
			Integer source = Integer.valueOf(spo[0]);
			Integer target = Integer.valueOf(spo[2]);			
			graph.addVertex(source);
			graph.addVertex(target);			
			graph.addEdge(source,target,new IntegerEdge(source,target, Integer.parseInt(spo[1])));
		}		
		
		GraphAgent graphAgent = new ExampleGraphAgent(graph);
		OracleAgent oracleAgent = new ExampleOracleAgent();
		
		Map<String,Integer> dictionary = readLableToIdDictionary();
		
		//Association finding.    
		List<Integer> queryEntities = new ArrayList<>();
		queryEntities.add(dictionary.get("Alice"));   //  7-Alice
		queryEntities.add(dictionary.get("Bob"));  // 8-Bob 
		queryEntities.add(dictionary.get("Chris"));  // 9-Chris	
		
		QUERYENTITIES = queryEntities;
		
		int diameter = 4;
		
		List<AssociationTree> associations = finder.discovery(graphAgent, oracleAgent, diameter, queryEntities);
		ASSOCIATIONSIZE = associations.size();
		
		Map<Integer,List<Integer>> typeStatements = readExampleTypeStatements();
		RelationChecker exampleRelationChecker = new ExampleRelationChecker();
		
		//Pattern Mining
		int tau = 2;
		List<PairBean> patterns = finder.partitionAndMiningPattern(associations.iterator(), tau , typeStatements, queryEntities, exampleRelationChecker);		
		return patterns;
	}


	/**
	 * 
	 * @return a map<instanceId,typeIds>.
	 * @throws IOException
	 */	
	private static Map<Integer, List<Integer>> readExampleTypeStatements() throws IOException {
		Map<Integer, List<Integer>> typeStatements = new TreeMap<>();
		List<String> allLine = Files.readAllLines(Paths.get("example/out_id_type_triples"), Charset.defaultCharset());
		for(String line : allLine){
			String[] spo = line.split(" ");
			Integer instance = Integer.valueOf(spo[0]);
			Integer type = Integer.valueOf(spo[2]);
			
			List<Integer> existedTypes = typeStatements.get(instance);
			if(existedTypes ==null){
				existedTypes = new ArrayList<>();
			}
			existedTypes.add(type);
			typeStatements.put(instance, existedTypes);
		}	
		return typeStatements;
	}
	/**
	 * 
	 * @return example Label-->Id dictionary.
	 */
	private static Map<String, Integer> readLableToIdDictionary() {
		Map<String,Integer> dictionary = new TreeMap<>();
		List<String> allLine = new ArrayList<>();
		try {
			allLine = Files.readAllLines(Paths.get("example/out_dict"), Charset.defaultCharset());
		} catch (IOException e) {
			e.printStackTrace();
		}
		for(String line : allLine){
			String[] spo = line.split(",");
			Integer id = Integer.valueOf(spo[0]);
			String uri = spo[1];
			dictionary.put(uri, id);
		}	
		return dictionary;
	}
}
