/*
 * Licensed to meast-summarize under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. IlluSnip licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package ws.meast.summarization.mps;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import name.dxliu.bean.PairBean;

/**
 * 
 * @author Daxin Liu
 *  Previously, a multiple dimensions knapsack problem is modeled to select favorable patterns. 
 *  To solve this knapsack problem, a greedy algorithm (select patterns that are of as less overlap as possible) is
 *  proposed. This class is the implementation of the greedy selector.
 *  
 *
 */
public class GreedyMultiPatternSelector {
	/**
	 * Compare two PairBean via their number of supporters.
	 */
	private static Comparator<PairBean> pairBeanComparator = new Comparator<PairBean>() {
		@Override
		public int compare(PairBean o1, PairBean o2) {//descending order.
			return o2.getValue().size()-o1.getValue().size();
		}
	};

	public static List<PairBean> selectByConstraint(List<Integer> queryEntities,final List<PairBean> inputPatternList,int K,int totalRelationNumber,double overlopThreshold,PatternDistance distanceCaculator){
		//step1  statistic overlaps.	
//		System.out.println("overlopThreshold:"+overlopThreshold);
//		long gedStart = System.currentTimeMillis();
//		int caculateRoundCounter = 0;
		List<PBContainer> inputPBC = new ArrayList<PBContainer>();
		for(int i=0,len=inputPatternList.size();i<len;i++){
			inputPBC.add(new PBContainer(inputPatternList.get(i)));
		}
		for(int i=0,len=inputPBC.size();i<len;i++){
			PBContainer outer = inputPBC.get(i);
			outer.index = i;
			outer.bitSet=new BitSet();	
			outer.bitSet.set(i);
			for(int j=0;j<i;j++){
				if(i==j){				
					continue;
				}
//				caculateRoundCounter++;
				double ged = distanceCaculator.getDistance(queryEntities,inputPBC.get(i).pb.getKey(), inputPBC.get(j).pb.getKey());
				if(ged<overlopThreshold){
					outer.bitSet.set(j);
					inputPBC.get(j).bitSet.set(i);
				}
			}			
		}
//		long gedEnd = System.currentTimeMillis();
//		System.out.println("Runing time of GED Caculate:"+ (gedEnd-gedStart)+"ms in" +caculateRoundCounter +" round");
		
		//step2 calculate priorities.
		for(PBContainer pbc:inputPBC){
			double freq = (pbc.pb.getValue().size()*1.0)/totalRelationNumber;
			double inf = 1;// infCaculator.getInfomativeness(pb.getKey());
			
			pbc.priority = freq*inf/(pbc.bitSet.cardinality()+1);
		}
		//step 4. sort
		
		List<PBContainer>inputPBCCopy = new ArrayList<PBContainer>(inputPBC);
		Collections.sort(inputPBCCopy,PBContainer.cmp);
		//step 5.select TopK
		BitSet selectedBeansConflicts = new BitSet();
		List<PairBean> result = new ArrayList<>();
		for(int i=0,len=inputPBCCopy.size();i<len;i++){			
			if(result.size()>=K) break;
			PBContainer pbc = inputPBCCopy.get(i);
			
			if(pbc.bitSet.intersects(selectedBeansConflicts)) continue;//if present pattern conflict with certain selected pattern.   如果当前Pattern与已经选中的有冲突，跳过
			result.add(pbc.pb);
			
			selectedBeansConflicts.set( pbc.index);
		}
		return result;
	}
	
	
	public static List<PairBean> selectByConstraintIgnoreOverlop(final List<PairBean> inputPatternList,int K){
		List<PairBean> inputPatternListCopy = new ArrayList<>(inputPatternList);
		Collections.sort(inputPatternListCopy,pairBeanComparator);
		
		int finalSize = inputPatternListCopy.size()>K?K:inputPatternListCopy.size();
		List<PairBean> result = new ArrayList<>(finalSize);
		for(int i = 0 ; i< finalSize;i++){
			result.add(inputPatternListCopy.get(i));			
		}
		return result;
	}
	
	//a container of pairbean for the convenience of solving MKP
	static class PBContainer{
		PairBean pb;
		public double priority = 0;
		public int index = -1;//for mps
		public BitSet  bitSet = null;
		
		public PBContainer(PairBean pb){
			this.pb = pb;
		}
		
		public static Comparator<PBContainer> cmp = new Comparator<PBContainer>(){
			@Override
			public int compare(PBContainer pbc1, PBContainer pbc2) {
				if(pbc2.priority==pbc1.priority)
					return 0;
				return	pbc2.priority-pbc1.priority>0?1:-1;
			}
			
		};
	}
	
}

